@@ -1,4 +1,5 @@
 /* application object */
+/* vim: set sw=2 et: */
 
 /*
  * Copyright (C) 2001 Havoc Pennington
@@ -38,6 +39,12 @@ G_BEGIN_DECLS
 typedef struct _WnckApplicationClass   WnckApplicationClass;
 typedef struct _WnckApplicationPrivate WnckApplicationPrivate;
 
+/**
+ * WnckApplication:
+ *
+ * The #WnckApplication struct contains only private fields and should not be
+ * directly accessed.
+ */
 struct _WnckApplication
 {
   GObject parent_instance;
@@ -54,6 +61,12 @@ struct _WnckApplicationClass
 
   /* icon changed */
   void (* icon_changed) (WnckApplication *app);
+  
+  /* Padding for future expansion */
+  void (* pad1) (void);
+  void (* pad2) (void);
+  void (* pad3) (void);
+  void (* pad4) (void);
 };
 
 GType wnck_application_get_type (void) G_GNUC_CONST;
@@ -62,7 +75,8 @@ WnckApplication* wnck_application_get (gulong xwindow);
 
 gulong wnck_application_get_xid (WnckApplication *app);
 
-GList* wnck_application_get_windows (WnckApplication *app);
+GList* wnck_application_get_windows   (WnckApplication *app);
+int    wnck_application_get_n_windows (WnckApplication *app);
 
 /* application_get_name, application_get_pid, etc.; prefer to read
  * properties straight off the group leader, and failing that, if the
@@ -75,6 +89,7 @@ int         wnck_application_get_pid       (WnckApplication *app);
 GdkPixbuf*  wnck_application_get_icon      (WnckApplication *app);
 GdkPixbuf*  wnck_application_get_mini_icon (WnckApplication *app);
 gboolean    wnck_application_get_icon_is_fallback (WnckApplication *app);
+const char* wnck_application_get_startup_id (WnckApplication *app);
 
 G_END_DECLS
 
