@@ -1,4 +1,5 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 2 -*- */
+/* vim: set sw=2 et: */
 /* pager object */
 
 /*
@@ -32,6 +33,28 @@
 #include "private.h"
 #include <math.h>
 
+/**
+ * SECTION:pager
+ * @short_description: a pager widget, showing the content of workspaces.
+ * @see_also: #WnckScreen
+ * @stability: Unstable
+ *
+ * A #WnckPager shows a miniature view of the workspaces, representing managed
+ * windows by small rectangles, and allows the user to initiate various window
+ * manager actions by manipulating these representations. The #WnckPager offers
+ * ways to move windows between workspaces and to change the current workspace.
+ *
+ * Alternatively, a #WnckPager can be configured to only show the names of the
+ * workspace instead of their contents.
+ *
+ * The #WnckPager is also responsible for setting the layout of the workspaces.
+ * Since only one application can be responsible for setting the layout on a
+ * screen, the #WnckPager automatically tries to obtain the manager selection
+ * for the screen and only sets the layout if it owns the manager selection.
+ * See wnck_pager_set_orientation() and wnck_pager_set_n_rows() to change the
+ * layout.
+ */
+
 #define N_SCREEN_CONNECTIONS 11
 
 struct _WnckPagerPrivate
@@ -62,6 +85,8 @@ struct _WnckPagerPrivate
   guint dnd_time; /* time of last event during dnd (for delayed workspace activation) */
 };
 
+G_DEFINE_TYPE (WnckPager, wnck_pager, GTK_TYPE_WIDGET);
+
 enum
 {
   dummy, /* remove this when you add more signals */
@@ -134,13 +159,12 @@ static void wnck_pager_queue_draw_workspace   (WnckPager   *pager,
 static void wnck_pager_queue_draw_window (WnckPager	   *pager,
 					  WnckWindow	   *window);
 
-static void wnck_pager_connect_screen    (WnckPager  *pager,
-                                          WnckScreen *screen);
+static void wnck_pager_connect_screen    (WnckPager  *pager);
 static void wnck_pager_connect_window    (WnckPager  *pager,
                                           WnckWindow *window);
 static void wnck_pager_disconnect_screen (WnckPager  *pager);
 
-static void wnck_pager_set_layout_hint   (WnckPager  *pager);
+static gboolean wnck_pager_set_layout_hint (WnckPager  *pager);
 
 static void wnck_pager_clear_drag (WnckPager *pager);
 static void wnck_pager_check_prelight (WnckPager *pager,
@@ -155,39 +179,6 @@ static GdkPixbuf* wnck_pager_get_background (WnckPager *pager,
 static AtkObject* wnck_pager_get_accessible (GtkWidget *widget);
 
 
-static gpointer parent_class;
-
-GType
-wnck_pager_get_type (void)
-{
-  static GType object_type = 0;
-
-  g_type_init ();
-  
-  if (!object_type)
-    {
-      const GTypeInfo object_info =
-      {
-        sizeof (WnckPagerClass),
-        (GBaseInitFunc) NULL,
-        (GBaseFinalizeFunc) NULL,
-        (GClassInitFunc) wnck_pager_class_init,
-        NULL,           /* class_finalize */
-        NULL,           /* class_data */
-        sizeof (WnckPager),
-        0,              /* n_preallocs */
-        (GInstanceInitFunc) wnck_pager_init,
-        NULL            /* value_table */
-      };
-      
-      object_type = g_type_register_static (GTK_TYPE_WIDGET,
-                                            "WnckPager",
-                                            &object_info, 0);
-    }
-  
-  return object_type;
-}
-
 static void
 wnck_pager_init (WnckPager *pager)
 {  
@@ -217,8 +208,6 @@ wnck_pager_class_init (WnckPagerClass *klass)
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
   GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
   
-  parent_class = g_type_class_peek_parent (klass);
-  
   object_class->finalize = wnck_pager_finalize;
 
   widget_class->realize = wnck_pager_realize;
@@ -247,8 +236,6 @@ wnck_pager_finalize (GObject *object)
 
   pager = WNCK_PAGER (object);
 
-  wnck_pager_disconnect_screen (pager);
-
   if (pager->priv->bg_cache)
     {
       g_object_unref (G_OBJECT (pager->priv->bg_cache));
@@ -263,7 +250,39 @@ wnck_pager_finalize (GObject *object)
 
   g_free (pager->priv);
   
-  G_OBJECT_CLASS (parent_class)->finalize (object);
+  G_OBJECT_CLASS (wnck_pager_parent_class)->finalize (object);
+}
+
+static void
+_wnck_pager_set_screen (WnckPager *pager)
+{
+  GdkScreen *gdkscreen;
+
+  gdkscreen = gtk_widget_get_screen (GTK_WIDGET (pager));
+  pager->priv->screen = wnck_screen_get (gdk_screen_get_number (gdkscreen));
+
+  if (!wnck_pager_set_layout_hint (pager))
+    {
+      _WnckLayoutOrientation orientation;
+
+      /* we couldn't set the layout on the screen. This means someone else owns
+       * it. Let's at least show the correct layout. */
+      _wnck_screen_get_workspace_layout (pager->priv->screen,
+                                         &orientation,
+                                         &pager->priv->n_rows,
+                                         NULL, NULL);
+
+      /* test in this order to default to horizontal in case there was in issue
+       * when fetching the layout */
+      if (orientation == WNCK_LAYOUT_ORIENTATION_VERTICAL)
+        pager->priv->orientation = GTK_ORIENTATION_VERTICAL;
+      else
+        pager->priv->orientation = GTK_ORIENTATION_HORIZONTAL;
+
+      gtk_widget_queue_resize (GTK_WIDGET (pager));
+    }
+
+  wnck_pager_connect_screen (pager);
 }
 
 static void
@@ -276,6 +295,8 @@ wnck_pager_realize (GtkWidget *widget)
 
   pager = WNCK_PAGER (widget);
 
+  /* do not call the parent class realize since we're doing things a bit
+   * differently here */
   GTK_WIDGET_SET_FLAGS (widget, GTK_REALIZED);
 
   attributes.window_type = GDK_WINDOW_CHILD;
@@ -299,7 +320,11 @@ wnck_pager_realize (GtkWidget *widget)
   widget->style = gtk_style_attach (widget->style, widget->window);
   gtk_style_set_background (widget->style, widget->window, GTK_STATE_NORMAL);
 
-  wnck_pager_set_layout_hint (pager);
+  /* connect to the screen of this pager. In theory, this will already have
+   * been done in wnck_pager_size_request() */
+  if (pager->priv->screen == NULL)
+    _wnck_pager_set_screen (pager);
+  g_assert (pager->priv->screen != NULL);
 }
 
 static void
@@ -315,7 +340,10 @@ wnck_pager_unrealize (GtkWidget *widget)
   wnck_screen_release_workspace_layout (pager->priv->screen,
                                         pager->priv->layout_manager_token);
   
-  GTK_WIDGET_CLASS (parent_class)->unrealize (widget);
+  wnck_pager_disconnect_screen (pager);
+  pager->priv->screen = NULL;
+
+  GTK_WIDGET_CLASS (wnck_pager_parent_class)->unrealize (widget);
 }
 
 static void
@@ -334,6 +362,11 @@ wnck_pager_size_request  (GtkWidget      *widget,
 
   pager = WNCK_PAGER (widget);
   
+  /* if we're not realized, we don't know about our screen yet */
+  if (pager->priv->screen == NULL)
+    _wnck_pager_set_screen (pager);
+  g_assert (pager->priv->screen != NULL);
+
   n_spaces = wnck_screen_get_workspace_count (pager->priv->screen);
 
   g_assert (pager->priv->n_rows > 0);
@@ -491,7 +524,8 @@ wnck_pager_size_allocate (GtkWidget      *widget,
       return;
     }
 
-  GTK_WIDGET_CLASS (parent_class)->size_allocate (widget, allocation);
+  GTK_WIDGET_CLASS (wnck_pager_parent_class)->size_allocate (widget,
+                                                             allocation);
 }
 
 static void
@@ -929,18 +963,18 @@ wnck_pager_draw_workspace (WnckPager    *pager,
   
   if (bg_pixbuf)
     {
-      gdk_pixbuf_render_to_drawable (bg_pixbuf,
-                                     widget->window,
-                                     widget->style->dark_gc[state],
-                                     0, 0,
-                                     rect->x, rect->y,
-                                     -1, -1,
-                                     GDK_RGB_DITHER_MAX,
-                                     0, 0);
+      gdk_draw_pixbuf (widget->window,
+                       widget->style->dark_gc[state],
+                       bg_pixbuf,
+                       0, 0,
+                       rect->x, rect->y,
+                       -1, -1,
+                       GDK_RGB_DITHER_MAX,
+                       0, 0);
     }
   else if (active_space)
     {
-      double vx, vy, vw, vh; /* viewport */
+      int vx, vy, vw, vh; /* viewport */
       double width_ratio, height_ratio;
       cairo_t *cr;
       
@@ -1409,8 +1443,8 @@ wnck_pager_drag_motion_leave (GtkWidget          *widget,
 static void
 wnck_drag_clean_up (WnckWindow     *window,
 		    GdkDragContext *context,
-		    gboolean	    clean_up_for_context_destroy,
-		    gboolean	    clean_up_for_window_destroy);
+		    gboolean	    clean_up_window,
+		    gboolean	    clean_up_context);
 
 static void
 wnck_drag_context_destroyed (gpointer  windowp,
@@ -1464,7 +1498,7 @@ wnck_update_drag_icon (WnckWindow     *window,
   gtk_drag_set_icon_pixmap (context, 
                             gdk_drawable_get_colormap (GDK_DRAWABLE (pixmap)),
 			    pixmap, NULL,
-			    rect.width / 4, 0);
+			    -2, -2);
 
   g_object_unref (pixmap);
 }
@@ -1489,19 +1523,11 @@ wnck_drag_source_destroyed (gpointer  contextp,
 static void
 wnck_drag_clean_up (WnckWindow     *window,
 		    GdkDragContext *context,
-		    gboolean	    clean_up_for_context_destroy,
-		    gboolean	    clean_up_for_window_destroy)
+		    gboolean	    clean_up_window,
+		    gboolean	    clean_up_context)
 {
-  if (clean_up_for_context_destroy)
+  if (clean_up_window)
     {
-      GtkWidget *drag_source;
-
-      drag_source = g_object_get_data (G_OBJECT (context),
-                                       "wnck-drag-source-widget");
-      if (drag_source)
-        g_object_weak_unref (G_OBJECT (drag_source),
-                             wnck_drag_source_destroyed, context);
-
       g_object_weak_unref (G_OBJECT (window),
                            wnck_drag_window_destroyed, context);
       if (g_signal_handlers_disconnect_by_func (window,
@@ -1509,9 +1535,19 @@ wnck_drag_clean_up (WnckWindow     *window,
 	g_assert_not_reached ();
     }
 
-  if (clean_up_for_window_destroy)
+  if (clean_up_context)
     {
-      g_object_steal_data (G_OBJECT (context), "wnck-drag-source-widget");
+      GtkWidget *drag_source;
+
+      drag_source = g_object_get_data (G_OBJECT (context),
+                                       "wnck-drag-source-widget");
+      if (drag_source)
+        {
+          g_object_steal_data (G_OBJECT (context), "wnck-drag-source-widget");
+          g_object_weak_unref (G_OBJECT (drag_source),
+                               wnck_drag_source_destroyed, context);
+        }
+
       g_object_weak_unref (G_OBJECT (context),
                            wnck_drag_context_destroyed, window);
     }
@@ -1664,22 +1700,35 @@ wnck_pager_focus (GtkWidget        *widget,
 
   pager = WNCK_PAGER (widget);
   
-  return GTK_WIDGET_CLASS (parent_class)->focus (widget, direction);
+  return GTK_WIDGET_CLASS (wnck_pager_parent_class)->focus (widget, direction);
 }
 
+/**
+ * wnck_pager_set_screen:
+ * @pager: a #WnckPager.
+ * @screen: a #WnckScreen.
+ *
+ * Does nothing.
+ *
+ * Since: 2.2
+ * Deprecated:
+ */
 void
 wnck_pager_set_screen (WnckPager  *pager,
 		       WnckScreen *screen)
 {
-  if (pager->priv->screen == screen)
-    return;
-
-  if (pager->priv->screen)
-    wnck_pager_disconnect_screen (pager);
-
-  wnck_pager_connect_screen (pager, screen);
 }
 
+/**
+ * wnck_pager_new:
+ * @screen: deprecated argument, can be %NULL.
+ *
+ * Creates a new #WnckPager. The #WnckPager will show the #WnckWorkspace of the
+ * #WnckScreen it is on.
+ *
+ * Return value: a newly created #WnckPager.
+ */
+/* TODO: when we break API again, remove the screen from here */
 GtkWidget*
 wnck_pager_new (WnckScreen *screen)
 {
@@ -1687,24 +1736,27 @@ wnck_pager_new (WnckScreen *screen)
   
   pager = g_object_new (WNCK_TYPE_PAGER, NULL);
 
-  wnck_pager_connect_screen (pager, screen);
-
   return GTK_WIDGET (pager);
 }
 
-static void
+static gboolean
 wnck_pager_set_layout_hint (WnckPager *pager)
 {
   int layout_rows;
   int layout_cols;
 
+  /* if we're not realized, we don't know about our screen yet */
+  if (pager->priv->screen == NULL)
+    _wnck_pager_set_screen (pager);
+  g_assert (pager->priv->screen != NULL);
+
   /* The visual representation of the pager doesn't
    * correspond to the layout of the workspaces
    * here. i.e. the user will not pay any attention
    * to the n_rows setting on this pager.
    */
   if (!pager->priv->show_all_workspaces)
-    return;
+    return FALSE;
 
   if (pager->priv->orientation == GTK_ORIENTATION_HORIZONTAL)
     {
@@ -1722,39 +1774,115 @@ wnck_pager_set_layout_hint (WnckPager *pager)
                                           pager->priv->layout_manager_token,
                                           layout_rows,
                                           layout_cols);
+
+  return (pager->priv->layout_manager_token != WNCK_NO_MANAGER_TOKEN);
 }
 
-void
+/**
+ * wnck_pager_set_orientation:
+ * @pager: a #WnckPager.
+ * @orientation: orientation to use for the layout of #WnckWorkspace on the
+ * #WnckScreen @pager is watching.
+ *
+ * Tries to change the orientation of the layout of #WnckWorkspace on the
+ * #WnckScreen @pager is watching. Since no more than one application should
+ * set this property of a #WnckScreen at a time, setting the layout is not
+ * guaranteed to work. 
+ *
+ * If @orientation is %GTK_ORIENTATION_HORIZONTAL, the #WnckWorkspace will be
+ * laid out in rows, with the first #WnckWorkspace in the top left corner.
+ *
+ * If @orientation is %GTK_ORIENTATION_VERTICAL, the #WnckWorkspace will be
+ * laid out in columns, with the first #WnckWorkspace in the top left corner.
+ *
+ * For example, if the layout contains one row, but the orientation of the
+ * layout is vertical, the #WnckPager will display a column of #WnckWorkspace.
+ *
+ * Return value: %TRUE if the layout of #WnckWorkspace has been successfully
+ * changed or did not need to be changed, %FALSE otherwise.
+ */
+gboolean
 wnck_pager_set_orientation (WnckPager     *pager,
                             GtkOrientation orientation)
 {
-  g_return_if_fail (WNCK_IS_PAGER (pager));
+  GtkOrientation old_orientation;
+  gboolean       old_orientation_is_valid;
+
+  g_return_val_if_fail (WNCK_IS_PAGER (pager), FALSE);
 
   if (pager->priv->orientation == orientation)
-    return;
+    return TRUE;
+
+  old_orientation = pager->priv->orientation;
+  old_orientation_is_valid = pager->priv->screen != NULL;
 
   pager->priv->orientation = orientation;
-  gtk_widget_queue_resize (GTK_WIDGET (pager));
 
-  wnck_pager_set_layout_hint (pager);
+  if (wnck_pager_set_layout_hint (pager))
+    {
+      gtk_widget_queue_resize (GTK_WIDGET (pager));
+      return TRUE;
+    }
+  else
+    {
+      if (old_orientation_is_valid)
+        pager->priv->orientation = old_orientation;
+      return FALSE;
+    }
 }
 
-void
+/**
+ * wnck_pager_set_n_rows:
+ * @pager: a #WnckPager.
+ * @n_rows: the number of rows to use for the layout of #WnckWorkspace on the
+ * #WnckScreen @pager is watching.
+ *
+ * Tries to change the number of rows in the layout of #WnckWorkspace on the
+ * #WnckScreen @pager is watching. Since no more than one application should
+ * set this property of a #WnckScreen at a time, setting the layout is not
+ * guaranteed to work. 
+ *
+ * Return value: %TRUE if the layout of #WnckWorkspace has been successfully
+ * changed or did not need to be changed, %FALSE otherwise.
+ */
+gboolean
 wnck_pager_set_n_rows (WnckPager *pager,
 		       int        n_rows)
 {
-  g_return_if_fail (WNCK_IS_PAGER (pager));
-  g_return_if_fail (n_rows > 0);
+  int      old_n_rows;
+  gboolean old_n_rows_is_valid;
+
+  g_return_val_if_fail (WNCK_IS_PAGER (pager), FALSE);
+  g_return_val_if_fail (n_rows > 0, FALSE);
 
   if (pager->priv->n_rows == n_rows)
-    return;
+    return TRUE;
+
+  old_n_rows = pager->priv->n_rows;
+  old_n_rows_is_valid = pager->priv->screen != NULL;
 
   pager->priv->n_rows = n_rows;
-  gtk_widget_queue_resize (GTK_WIDGET (pager));
 
-  wnck_pager_set_layout_hint (pager);
+  if (wnck_pager_set_layout_hint (pager))
+    {
+      gtk_widget_queue_resize (GTK_WIDGET (pager));
+      return TRUE;
+    }
+  else
+    {
+      if (old_n_rows_is_valid)
+        pager->priv->n_rows = old_n_rows;
+      return FALSE;
+    }
 }
 
+/**
+ * wnck_pager_set_display_mode:
+ * @pager: a #WnckPager.
+ * @mode: a display mode.
+ *
+ * Sets the display mode for @pager to @mode.
+ */
 void
 wnck_pager_set_display_mode (WnckPager            *pager,
 			     WnckPagerDisplayMode  mode)
@@ -1768,6 +1896,14 @@ wnck_pager_set_display_mode (WnckPager            *pager,
   gtk_widget_queue_resize (GTK_WIDGET (pager));
 }
 
+/**
+ * wnck_pager_set_show_all:
+ * @pager: a #WnckPager.
+ * @show_all_workspaces: whether to display all #WnckWorkspace in @pager.
+ *
+ * Sets @pager to display all #WnckWorkspace or not, according to
+ * @show_all_workspaces.
+ */
 void
 wnck_pager_set_show_all (WnckPager *pager,
 			 gboolean   show_all_workspaces)
@@ -1783,6 +1919,17 @@ wnck_pager_set_show_all (WnckPager *pager,
   gtk_widget_queue_resize (GTK_WIDGET (pager));
 }
 
+/**
+ * wnck_pager_set_shadow_type:
+ * @pager: a #WnckPager.
+ * @shadow_type: a shadow type.
+ *
+ * Sets the shadow type for @pager to @shadow_type. The main use of this
+ * function is proper integration of #WnckPager in panels with non-system
+ * backgrounds.
+ *
+ * Since: 2.2
+ */
 void
 wnck_pager_set_shadow_type (WnckPager *   pager,
 			    GtkShadowType shadow_type)
@@ -1798,6 +1945,7 @@ wnck_pager_set_shadow_type (WnckPager *   pager,
 
 static void
 active_window_changed_callback    (WnckScreen      *screen,
+                                   WnckWindow      *previous_window,
                                    gpointer         data)
 {
   WnckPager *pager = WNCK_PAGER (data);
@@ -1806,6 +1954,7 @@ active_window_changed_callback    (WnckScreen      *screen,
 
 static void
 active_workspace_changed_callback (WnckScreen      *screen,
+                                   WnckWorkspace   *previous_workspace,
                                    gpointer         data)
 {
   WnckPager *pager = WNCK_PAGER (data);
@@ -1954,17 +2103,17 @@ viewports_changed_callback (WnckWorkspace *space,
 }
 
 static void
-wnck_pager_connect_screen (WnckPager  *pager,
-                           WnckScreen *screen)
+wnck_pager_connect_screen (WnckPager *pager)
 {
   int i;
   guint *c;
   GList *tmp;
+  WnckScreen *screen;
   
-  g_return_if_fail (pager->priv->screen == NULL);
-  
-  pager->priv->screen = screen;
+  g_return_if_fail (pager->priv->screen != NULL);
 
+  screen = pager->priv->screen;
+  
   for (tmp = wnck_screen_get_windows (screen); tmp; tmp = tmp->next)
     {
       wnck_pager_connect_window (pager, WNCK_WINDOW (tmp->data));
@@ -2087,8 +2236,6 @@ wnck_pager_disconnect_screen (WnckPager  *pager)
       space = wnck_screen_get_workspace (pager->priv->screen, i);
       g_signal_handlers_disconnect_by_func (space, G_CALLBACK (workspace_name_changed_callback), pager);
     }
-
-  pager->priv->screen = NULL;
 }
 
 static void
@@ -2204,7 +2351,7 @@ wnck_pager_get_accessible (GtkWidget *widget)
         }
       first_time = FALSE;
     }
-  return GTK_WIDGET_CLASS (parent_class)->get_accessible (widget);
+  return GTK_WIDGET_CLASS (wnck_pager_parent_class)->get_accessible (widget);
 }
 
 int 
