@@ -1,4 +1,5 @@
 /* workspace object */
+/* vim: set sw=2 et: */
 
 /*
  * Copyright (C) 2001 Havoc Pennington
@@ -29,6 +30,39 @@
 #include "private.h"
 #include <string.h>
 
+/**
+ * SECTION:workspace
+ * @short_description: an object representing a workspace.
+ * @see_also: #WnckScreen
+ * @stability: Unstable
+ *
+ * The #WnckWorkspace represents what is called <ulink
+ * url="http://standards.freedesktop.org/wm-spec/wm-spec-latest.html&num;largedesks">virtual
+ * desktops</ulink> in the <ulink
+ * url="http://standards.freedesktop.org/wm-spec/wm-spec-latest.html">EWMH</ulink>.
+ * A workspace is a virtualization of a #WnckScreen<!-- -->: only one workspace
+ * can be shown on a #WnckScreen at a time. It makes it possible, for example,
+ * to put windows on different workspaces to organize them.
+ *
+ * If the #WnckWorkspace size is bigger that the #WnckScreen size, the
+ * workspace contains a viewport. Viewports are defined in the <ulink
+ * url="http://standards.freedesktop.org/wm-spec/wm-spec-latest.html&num;id2457064">large
+ * desktops</ulink> section of the <ulink
+ * url="http://standards.freedesktop.org/wm-spec/wm-spec-latest.html">EWMH</ulink>.
+ * The notion of workspaces and viewports are quite similar, and generally both
+ * are not used at the same time: there are generally either multiple
+ * workspaces with no viewport, or one workspace with a viewport. libwnck
+ * supports all situations, even multiple workspaces with viewports.
+ *
+ * Workspaces are organized according to a layout set on the #WnckScreen. See
+ * wnck_screen_calc_workspace_layout(), wnck_screen_try_set_workspace_layout(),
+ * wnck_screen_release_workspace_layout() and
+ * wnck_screen_free_workspace_layout() for more information about the layout.
+ *
+ * The #WnckWorkspace objects are always owned by libwnck and must not be
+ * referenced or unreferenced.
+ */
+
 struct _WnckWorkspacePrivate
 {
   WnckScreen *screen;
@@ -39,6 +73,8 @@ struct _WnckWorkspacePrivate
   gboolean is_virtual;
 };
 
+G_DEFINE_TYPE (WnckWorkspace, wnck_workspace, G_TYPE_OBJECT);
+
 enum {
   NAME_CHANGED,
   LAST_SIGNAL
@@ -51,40 +87,8 @@ static void wnck_workspace_finalize    (GObject        *object);
 
 static void emit_name_changed (WnckWorkspace *space);
 
-static gpointer parent_class;
 static guint signals[LAST_SIGNAL] = { 0 };
 
-GType
-wnck_workspace_get_type (void)
-{
-  static GType object_type = 0;
-
-  g_type_init ();
-  
-  if (!object_type)
-    {
-      static const GTypeInfo object_info =
-      {
-        sizeof (WnckWorkspaceClass),
-        (GBaseInitFunc) NULL,
-        (GBaseFinalizeFunc) NULL,
-        (GClassInitFunc) wnck_workspace_class_init,
-        NULL,           /* class_finalize */
-        NULL,           /* class_data */
-        sizeof (WnckWorkspace),
-        0,              /* n_preallocs */
-        (GInstanceInitFunc) wnck_workspace_init,
-        NULL            /* value_table */
-      };
-      
-      object_type = g_type_register_static (G_TYPE_OBJECT,
-                                            "WnckWorkspace",
-                                            &object_info, 0);
-    }
-  
-  return object_type;
-}
-
 static void
 wnck_workspace_init (WnckWorkspace *workspace)
 {
@@ -98,10 +102,14 @@ wnck_workspace_class_init (WnckWorkspaceClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
   
-  parent_class = g_type_class_peek_parent (klass);
-  
   object_class->finalize = wnck_workspace_finalize;
 
+  /**
+   * WnckWorkspace::name-changed:
+   * @space: the #WnckWorkspace which emitted the signal.
+   *
+   * Emitted when the name of @space changes.
+   */
   signals[NAME_CHANGED] =
     g_signal_new ("name_changed",
                   G_OBJECT_CLASS_TYPE (object_class),
@@ -123,35 +131,35 @@ wnck_workspace_finalize (GObject *object)
   
   g_free (workspace->priv);
   
-  G_OBJECT_CLASS (parent_class)->finalize (object);
+  G_OBJECT_CLASS (wnck_workspace_parent_class)->finalize (object);
 }
 
 /**
  * wnck_workspace_get_number:
- * @space: a #WnckWorkspace
+ * @space: a #WnckWorkspace.
  * 
+ * Returns the index of @space on the #WnckScreen to which it belongs. The
+ * first workspace has an index of 0.
  * 
- * 
- * Return value: get the index of the workspace
+ * Return value: the index of @space on its #WnckScreen, or -1 on errors.
  **/
 int
 wnck_workspace_get_number (WnckWorkspace *space)
 {
-  g_return_val_if_fail (WNCK_IS_WORKSPACE (space), 0);
+  g_return_val_if_fail (WNCK_IS_WORKSPACE (space), -1);
   
   return space->priv->number;
 }
 
 /**
  * wnck_workspace_get_name:
- * @space: a #WnckWorkspace
+ * @space: a #WnckWorkspace.
  * 
- * Gets the name that should be used to refer to the workspace
- * in the user interface. If the user hasn't set a special name,
- * will be something like "Workspace 3" - otherwise whatever name
- * the user set.
+ * Returns the human-readable name that should be used to refer to @space. If
+ * the user has not set a special name, a fallback like "Workspace 3" will be
+ * used.
  * 
- * Return value: workspace name, never %NULL 
+ * Return value: the name of @space.
  **/
 const char*
 wnck_workspace_get_name (WnckWorkspace *space)
@@ -163,11 +171,12 @@ wnck_workspace_get_name (WnckWorkspace *space)
 
 /**
  * wnck_workspace_change_name:
- * @space: a #WnckWorkspace
- * @name: new workspace name
+ * @space: a #WnckWorkspace.
+ * @name: new name for @space.
  *
- * Try changing the name of the workspace.
+ * Changes the name of @space.
  * 
+ * Since: 2.2
  **/
 void
 wnck_workspace_change_name (WnckWorkspace *space,
@@ -183,10 +192,15 @@ wnck_workspace_change_name (WnckWorkspace *space,
 
 /**
  * wnck_workspace_activate:
- * @space: a #WnckWorkspace
+ * @space: a #WnckWorkspace.
+ * @timestamp: the X server timestamp of the user interaction event that caused
+ * this call to occur.
  * 
- * Ask window manager to make @space the active workspace.
+ * Asks the window manager to make @space the active workspace. The window
+ * manager may decide to refuse the request (to not steal the focus if there is
+ * a more recent user activity, for example).
  * 
+ * Since: 2.10 (@timestamp argument was missing in earlier versions)
  **/
 void
 wnck_workspace_activate (WnckWorkspace *space,
@@ -288,32 +302,94 @@ _wnck_workspace_set_viewport (WnckWorkspace *space,
     return FALSE;
 }
 
+/**
+ * wnck_workspace_get_width:
+ * @space: a #WnckWorkspace.
+ *
+ * Returns the width of @space.
+ *
+ * Returns: the width of @space.
+ *
+ * Since: 2.4
+ */
 int
 wnck_workspace_get_width (WnckWorkspace *space)
 {
+  g_return_val_if_fail (WNCK_IS_WORKSPACE (space), 0);
+
   return space->priv->width;
 }
 
+/**
+ * wnck_workspace_get_height:
+ * @space: a #WnckWorkspace.
+ *
+ * Returns the height of @space.
+ *
+ * Returns: the height of @space.
+ *
+ * Since: 2.4
+ */
 int
 wnck_workspace_get_height (WnckWorkspace *space)
 {
+  g_return_val_if_fail (WNCK_IS_WORKSPACE (space), 0);
+
   return space->priv->height;
 }
 
+/**
+ * wnck_workspace_get_viewport_x:
+ * @space: a #WnckWorkspace.
+ *
+ * Returns the X coordinate of the viewport in @space.
+ *
+ * Returns: the X coordinate of the viewport in @space, or 0 if @space does not
+ * contain a viewport.
+ *
+ * Since: 2.4
+ */
 int
 wnck_workspace_get_viewport_x (WnckWorkspace *space)
 {
+  g_return_val_if_fail (WNCK_IS_WORKSPACE (space), 0);
+
   return space->priv->viewport_x;
 }
 
+/**
+ * wnck_workspace_get_viewport_y:
+ * @space: a #WnckWorkspace.
+ *
+ * Returns the Y coordinate of the viewport in @space.
+ *
+ * Returns: the Y coordinate of the viewport in @space, or 0 if @space does not
+ * contain a viewport.
+ *
+ * Since: 2.4
+ */
 int
 wnck_workspace_get_viewport_y (WnckWorkspace *space)
 {
+  g_return_val_if_fail (WNCK_IS_WORKSPACE (space), 0);
+
   return space->priv->viewport_y;
 }
 
+/**
+ * wnck_workspace_is_virtual:
+ * @space: a #WnckWorkspace.
+ *
+ * Returns whether @space contains a viewport.
+ *
+ * Returns: %TRUE if @space contains a viewport, %FALSE otherwise.
+ *
+ * Since: 2.4
+ */
 gboolean
 wnck_workspace_is_virtual (WnckWorkspace *space)
 {
+  g_return_val_if_fail (WNCK_IS_WORKSPACE (space), FALSE);
+
   return space->priv->is_virtual;
 }
