@@ -1,4 +1,5 @@
 /* Xlib utils */
+/* vim: set sw=2 et: */
 
 /*
  * Copyright (C) 2001 Havoc Pennington
@@ -245,11 +246,13 @@ text_property_to_utf8 (const XTextProperty *prop)
                                           &list);
 
   if (count == 0)
-    return NULL;
+    retval = NULL;
+  else
+    {
+      retval = list[0];
+      list[0] = g_strdup (""); /* something to free */
+    }
 
-  retval = list[0];
-  list[0] = g_strdup (""); /* something to free */
-  
   g_strfreev (list);
 
   return retval;
@@ -293,7 +296,7 @@ _wnck_get_string_property_latin1 (Window  xwindow,
   int format;
   gulong nitems;
   gulong bytes_after;
-  guchar *str;
+  gchar *str;
   int err, result;
   char *retval;
   
@@ -331,7 +334,7 @@ _wnck_get_utf8_property (Window  xwindow,
   int format;
   gulong nitems;
   gulong bytes_after;
-  guchar *val;
+  gchar *val;
   int err, result;
   char *retval;
   Atom utf8_string;
@@ -625,7 +628,7 @@ _wnck_set_utf8_list (Window   xwindow,
 		   xwindow,
                    atom,
 		   utf8_string, 8, PropModeReplace,
-		   flattened->str, flattened->len);
+		   (guchar *) flattened->str, flattened->len);
   
   _wnck_error_trap_pop ();
 
@@ -1138,6 +1141,25 @@ _wnck_get_name (Window xwindow)
   return name;
 }
 
+char*
+_wnck_get_icon_name (Window xwindow)
+{
+  char *name;
+  
+  name = _wnck_get_utf8_property (xwindow,
+                                  _wnck_atom_get ("_NET_WM_VISIBLE_ICON_NAME"));
+
+  if (name == NULL)
+    name = _wnck_get_utf8_property (xwindow,
+                                    _wnck_atom_get ("_NET_WM_ICON_NAME"));
+
+  if (name == NULL)
+    name = _wnck_get_text_property (xwindow,
+                                    XA_WM_ICON_NAME);
+
+  return name;
+}
+
 static char*
 latin1_to_utf8 (const char *latin1)
 {
@@ -1209,6 +1231,41 @@ _wnck_get_wmclass (Window xwindow,
     }
 }
 
+gboolean
+_wnck_get_frame_extents (Window  xwindow,
+                         int    *left_frame,
+                         int    *right_frame,
+                         int    *top_frame,
+                         int    *bottom_frame)
+{
+  gulong   *p_size;
+  int       n_size;
+  gboolean  retval;
+
+  retval = FALSE;
+  p_size = NULL;
+  n_size = 0;
+
+  _wnck_get_cardinal_list (xwindow,
+                           _wnck_atom_get ("_NET_FRAME_EXTENTS"),
+                           &p_size, &n_size);
+
+  if (p_size != NULL && n_size == 4)
+    {
+      *left_frame   = p_size[0];
+      *right_frame  = p_size[1];
+      *top_frame    = p_size[2];
+      *bottom_frame = p_size[3];
+
+      retval = TRUE;
+    }
+
+  if (p_size != NULL)
+    g_free (p_size);
+
+  return retval;
+}
+
 void
 _wnck_select_input (Window xwindow,
                     int    mask)
@@ -2214,7 +2271,8 @@ _wnck_get_window_geometry (Screen *screen,
                            int    *widthp,
                            int    *heightp)
 {
-  int x, y, width, height, bw, depth;
+  int x, y;
+  unsigned int width, height, bw, depth;
   Window root_window;
 
   width = 1;
@@ -2318,24 +2376,6 @@ _wnck_set_icon_geometry  (Window xwindow,
   _wnck_error_trap_pop ();
 }
 
-void
-_wnck_set_dock_type_hint (Window xwindow)
-{
-  Atom atom;
-
-  atom = _wnck_atom_get ("_NET_WM_WINDOW_TYPE_DOCK");
-  
-  _wnck_error_trap_push ();
-
-  XChangeProperty (gdk_display,
-                   xwindow, 
-                   _wnck_atom_get ("_NET_WM_WINDOW_TYPE"),
-		   XA_ATOM, 32, PropModeReplace,
-		   (guchar *)&atom, 1);
-
-  _wnck_error_trap_pop ();
-}
-
 /* orientation of pager */
 #define _NET_WM_ORIENTATION_HORZ 0
 #define _NET_WM_ORIENTATION_VERT 1
@@ -2410,21 +2450,20 @@ timestamp_predicate (Display *display,
  * Return value: the time stamp.
  **/
 static Time
-get_server_time (Display *display,
-		 Window   window)
+get_server_time (Window window)
 {
   unsigned char c = 'a';
   XEvent xevent;
   TimeStampInfo info;
 
-  info.timestamp_prop_atom = XInternAtom  (display, "_TIMESTAMP_PROP", False);
+  info.timestamp_prop_atom = _wnck_atom_get ("_TIMESTAMP_PROP");
   info.window = window;
 
-  XChangeProperty (display, window,
+  XChangeProperty (gdk_display, window,
 		   info.timestamp_prop_atom, info.timestamp_prop_atom,
 		   8, PropModeReplace, &c, 1);
 
-  XIfEvent (display, &xevent,
+  XIfEvent (gdk_display, &xevent,
 	    timestamp_predicate, (XPointer)&info);
 
   return xevent.xproperty.time;
@@ -2442,19 +2481,35 @@ typedef struct
 static GSList *layout_managers = NULL;
 static int next_token = 1;
 
+static void
+_wnck_free_layout_manager (LayoutManager *lm)
+{
+  XDestroyWindow (gdk_display, lm->window);
+  g_free (lm);
+
+  layout_managers = g_slist_remove (layout_managers, lm);
+}
+
 int
 _wnck_try_desktop_layout_manager (Screen *xscreen,
                                   int     current_token)
 {
+  Atom selection_atom;
+  Window owner;
   GSList *tmp;
   int number;
   Time timestamp;
   XClientMessageEvent xev;  
   char buffer[256];
   LayoutManager *lm;
-  
+
   number = XScreenNumberOfScreen (xscreen);
   
+  sprintf (buffer, "_NET_DESKTOP_LAYOUT_S%d", number);
+  selection_atom = _wnck_atom_get (buffer);
+
+  owner = XGetSelectionOwner (gdk_display, selection_atom);
+  
   tmp = layout_managers;
   while (tmp != NULL)
     {
@@ -2463,7 +2518,15 @@ _wnck_try_desktop_layout_manager (Screen *xscreen,
       if (number == lm->screen_number)
         {
           if (current_token == lm->token)
-            return current_token; /* we still have the selection */
+            {
+              if (owner == lm->window)
+                return current_token; /* we still have the selection */
+              else
+                { /* we lost the selection */
+                  _wnck_free_layout_manager (lm);
+                  break;
+                }
+            }
           else
             return WNCK_NO_MANAGER_TOKEN; /* someone else has it */
         }
@@ -2471,7 +2534,10 @@ _wnck_try_desktop_layout_manager (Screen *xscreen,
       tmp = tmp->next;
     }
   
-  /* No one in-process has the selection at the moment */
+  if (owner != None)
+    return WNCK_NO_MANAGER_TOKEN; /* someone else has the selection */
+
+  /* No one has the selection at the moment */
 
   lm = g_new0 (LayoutManager, 1);
 
@@ -2479,9 +2545,8 @@ _wnck_try_desktop_layout_manager (Screen *xscreen,
   lm->token = next_token;
   ++next_token;
 
-  sprintf (buffer, "_NET_DESKTOP_LAYOUT_S%d", number);
-  lm->selection_atom = XInternAtom (gdk_display, buffer, False);
-  lm->manager_atom = XInternAtom (gdk_display, "MANAGER", False);
+  lm->selection_atom = selection_atom;
+  lm->manager_atom = _wnck_atom_get ("MANAGER");
 
   lm->window = XCreateSimpleWindow (gdk_display,
                                     RootWindowOfScreen (xscreen),
@@ -2490,7 +2555,7 @@ _wnck_try_desktop_layout_manager (Screen *xscreen,
                                     WhitePixel (gdk_display, number));
 
   XSelectInput (gdk_display, lm->window, PropertyChangeMask);
-  timestamp = get_server_time (gdk_display, lm->window);
+  timestamp = get_server_time (lm->window);
 
   XSetSelectionOwner (gdk_display, lm->selection_atom,
 		      lm->window, timestamp);
@@ -2542,12 +2607,18 @@ _wnck_release_desktop_layout_manager (Screen *xscreen,
         {
           if (current_token == lm->token)
             {
-              XDestroyWindow (gdk_display,
-                              lm->window);
+              /* release selection ownership */
+              if (XGetSelectionOwner (gdk_display, lm->selection_atom) !=
+                  lm->window)
+                {
+                  Time timestamp;
 
-              g_free (lm);
-              layout_managers = g_slist_remove (layout_managers, lm);
-              
+                  timestamp = get_server_time (lm->window);
+                  XSetSelectionOwner (gdk_display, lm->selection_atom,
+                                      None, timestamp);
+                }
+
+              _wnck_free_layout_manager (lm);
               return;
             }
         }
@@ -2573,12 +2644,7 @@ _wnck_desktop_layout_manager_process_event (XEvent *xev)
       if (xev->xany.window == lm->window &&
           xev->xselectionclear.selection == lm->selection_atom)
         {
-          XDestroyWindow (gdk_display,
-                          lm->window);
-          
-          g_free (lm);
-          layout_managers = g_slist_remove (layout_managers, lm);
-          
+          _wnck_free_layout_manager (lm);
           return TRUE;
         }
       
@@ -2587,51 +2653,3 @@ _wnck_desktop_layout_manager_process_event (XEvent *xev)
 
   return FALSE;
 }
-
-/* stock icon code Copyright (C) 2002 Jorn Baayen <jorn@nl.linux.org> */
-typedef struct
-{
-  char *stock_id;
-  const guint8 *icon_data;
-} StockIcon;
-
-void
-_wnck_stock_icons_init (void)
-{
-  GtkIconFactory *factory;
-  int i;
-  static gboolean done = FALSE;
-
-  StockIcon items[] =
-  {
-    { WNCK_STOCK_DELETE,   stock_delete_data   },
-    { WNCK_STOCK_MINIMIZE, stock_minimize_data },
-    { WNCK_STOCK_MAXIMIZE, stock_maximize_data }
-  };
-
-  if (done)
-    return;
-
-  done = TRUE;
-  
-  factory = gtk_icon_factory_new ();
-  gtk_icon_factory_add_default (factory);
-
-  for (i = 0; i < (gint) G_N_ELEMENTS (items); i++)
-    {
-      GtkIconSet *icon_set;
-      GdkPixbuf *pixbuf;
-
-      pixbuf = gdk_pixbuf_new_from_inline (-1, items[i].icon_data,
-					   FALSE,
-					   NULL);
-
-      icon_set = gtk_icon_set_new_from_pixbuf (pixbuf);
-      gtk_icon_factory_add (factory, items[i].stock_id, icon_set);
-      gtk_icon_set_unref (icon_set);
-		
-      g_object_unref (G_OBJECT (pixbuf));
-    }
-
-  g_object_unref (G_OBJECT (factory));
-}
